import 'dart:async';
import 'dart:io';

import 'package:build/build.dart';
import 'package:path/path.dart';

/// This builder create one file: lib/src/flags.dart, it run only after all calls
/// to [AssetToCacheBuilder.build] are finished.
///
/// It triggered only if `flags.dart` file didn't exist, for this reason
/// [AssetToCacheBuilder.build] will always delete this file.
///
/// It combine all files generated by [AssetToCacheBuilder].
class FlagsFileBuilder implements Builder {
  FlagsFileBuilder();

  static AssetId _allFileOutput(BuildStep buildStep) {
    return AssetId(
      buildStep.inputId.package,
      join('lib', 'src', 'flags.dart'),
    );
  }

  @override
  Map<String, List<String>> get buildExtensions => {
        r'$lib$': const ['src/flags.dart']
      };

  @override
  FutureOr<void> build(BuildStep buildStep) async {
    final flagFile = File('lib/src/flags.dart');
    if (flagFile.existsSync()) {
      flagFile.deleteSync();
    }

    // make sure Builders never run in user package even if user
    // import package from git or other sources
    // (this doesn't needed for pub.dev, because of .pubignore)
    if (buildStep.inputId.package != 'circle_flags') {
      return;
    }

    final output = _allFileOutput(buildStep);

    print("Writing ${output.path}");
    final fileHeader = '''
// GENERATED FILE, timestamp: ${DateTime.now()}
// Regenerate with: dart run build_runner build

import 'package:circle_flags/circle_flags.dart';

/// List of available flags(iso codes) for [CircleFlag].
abstract class Flags {
''';

    var list = '''
  static const values = <String>[
''';
    var newFileContent = '';
    for (final file
        in Directory(".dart_tool/build/generated/circle_flags/assets/svg/")
            .listSync()
            .toList()
          ..sort((a, b) => a.path.compareTo(b.path))) {
      if (file is File) {
        newFileContent += file.readAsStringSync();
        var baseName = file.path.split('/').last;
        baseName = baseName.substring(0, baseName.length - 9); // cut .part.txt
        list += "    ${baseName.toUpperCase().replaceAll('-', '_')},\n";
      }
    }

    list += "  ];\n";
    newFileContent += "}\n";
    return buildStep.writeAsString(output, fileHeader + list + newFileContent);
  }
}
